﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.IO;
using Syncfusion.Windows.Controls.RichTextBoxAdv;

namespace SFRtbAdvV17
{
    /// <summary>
    /// Represents the extension class for SfRichTextBoxAdv.
    /// </summary>
    public class SfRichTextBoxAdvExtension : SfRichTextBoxAdv
    {
        #region Fields
        bool skipUpdating = false;
        #endregion

        #region Properties
        /// <summary>
        /// Gets or Sets the html text.
        /// </summary>
        public string HtmlText
        {
            get
            {
                return (string)GetValue(HtmlTextProperty);
            }
            set
            {
                SetValue(HtmlTextProperty, value);
            }
        }
        public string RtfText
        {
            get
            {
                return (string)GetValue(RtfTextProperty);
            }
            set
            {
                SetValue(RtfTextProperty, value);
            }
        }
        public string PlainText
        {
            get
            {
                return (string)GetValue(PlainTextProperty);
            }
            set
            {
                SetValue(PlainTextProperty, value);
            }
        }
        #endregion

        #region Constructor
        /// <summary>
        /// Initializes the instance of SfRichTextBoxAdvExtension class.
        /// </summary>
        public SfRichTextBoxAdvExtension()
        {
            // Wires the ContentChanged event.
            this.ContentChanged += RTE_ContentChanged;
        }
        #endregion

        #region Static Dependency Properties
        /// <summary>
        /// Using as a backing store for HtmlText dependency property to enable styling, animation etc.
        /// </summary>
        public static readonly DependencyProperty HtmlTextProperty = DependencyProperty.Register("HtmlText", typeof(string), typeof(SfRichTextBoxAdvExtension), new PropertyMetadata(string.Empty, new PropertyChangedCallback(OnHtmlTextChanged)));
        public static readonly DependencyProperty RtfTextProperty = DependencyProperty.Register("RtfText", typeof(string), typeof(SfRichTextBoxAdvExtension), new PropertyMetadata(string.Empty, new PropertyChangedCallback(OnRtfTextChanged)));
        public static readonly DependencyProperty PlainTextProperty = DependencyProperty.Register("PlainText", typeof(string), typeof(SfRichTextBoxAdvExtension), new PropertyMetadata(string.Empty, new PropertyChangedCallback(OnPlainTextChanged)));
        #endregion

        #region Static Events
        /// <summary>
        /// Called when html text changed.
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="e"></param>
        private static void OnHtmlTextChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            SfRichTextBoxAdvExtension richTextBox = (SfRichTextBoxAdvExtension)obj;
            //Update the document with the HTML.
            richTextBox.UpdateDocumentHtml((string)e.NewValue);
        }
        private static void OnRtfTextChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            SfRichTextBoxAdvExtension richTextBox = (SfRichTextBoxAdvExtension)obj;
            //Update the document with the Rtf.
            richTextBox.UpdateDocumentRtf((string)e.NewValue);
        }
        private static void OnPlainTextChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            SfRichTextBoxAdvExtension richTextBox = (SfRichTextBoxAdvExtension)obj;
            //Update the document with the Plain.
            richTextBox.UpdateDocumentPlain((string)e.NewValue);
        }
        #endregion

        #region Events
        /// <summary>
        /// Called when content changes in SfRichTextBoxAdv.
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="args"></param>
        void RTE_ContentChanged(object obj, ContentChangedEventArgs args)
        {
            if (this.Document != null)
            {
                // To skip internal updation of document on setting HtmlText property.
                skipUpdating = true;
                Stream stream = new MemoryStream();
                // Saves the document as Html Stream.
                this.Save(stream, FormatType.Html);
                stream.Position = 0;
                // Reads the stream and assigned the string to HtmlText property
                using (StreamReader reader = new StreamReader(stream))
                {
                    this.HtmlText = reader.ReadToEnd();
                }

                Stream RtfStream = new MemoryStream();
                // Saves the document as Rtf Stream.
                this.Save(RtfStream, FormatType.Rtf);
                RtfStream.Position = 0;
                // Reads the stream and assigned the string to RtfText property
                using (StreamReader reader = new StreamReader(RtfStream))
                {
                    this.RtfText = reader.ReadToEnd();
                }

                Stream PlainStream = new MemoryStream();
                // Saves the document as Plain Stream.
                this.Save(PlainStream, FormatType.Txt);
                PlainStream.Position = 0;
                // Reads the stream and assigned the string to PlainText property
                using (StreamReader reader = new StreamReader(PlainStream))
                {
                    this.PlainText = reader.ReadToEnd();
                }

                skipUpdating = false;
            }
        }
        #endregion

        #region Implementation
        /// <summary>
        /// Updates the document.
        /// </summary>
        /// <param name="htmlText"></param>
        private void UpdateDocumentHtml(string htmlText)
        {
            // If Html text property is set internally means, skip updating the document.
            if (!skipUpdating && !string.IsNullOrEmpty(htmlText))
            {
                Stream stream = new MemoryStream();
                // Convert the Html string to byte array.
                byte[] bytes = Encoding.UTF8.GetBytes(htmlText);
                // Writes the byte array to stream.
                stream.Write(bytes, 0, bytes.Length);
                stream.Position = 0;
                //Load the Html stream.
                Load(stream, FormatType.Html);
            }
        }
        private void UpdateDocumentRtf(string RtfText)
        {
            // If Rtf text property is set internally means, skip updating the document.
            if (!skipUpdating && !string.IsNullOrEmpty(RtfText))
            {
                Stream stream = new MemoryStream();
                // Convert the Rtf string to byte array.
                byte[] bytes = Encoding.UTF8.GetBytes(RtfText);
                // Writes the byte array to stream.
                stream.Write(bytes, 0, bytes.Length);
                stream.Position = 0;
                //Load the Rtf stream.
                Load(stream, FormatType.Rtf);
            }
        }

        private void UpdateDocumentPlain(string PlainText)
        {
            // If Plain text property is set internally means, skip updating the document.
            if (!skipUpdating && !string.IsNullOrEmpty(PlainText))
            {
                Stream stream = new MemoryStream();
                // Convert the Plain string to byte array.
                byte[] bytes = Encoding.UTF8.GetBytes(PlainText);
                // Writes the byte array to stream.
                stream.Write(bytes, 0, bytes.Length);
                stream.Position = 0;
                //Load the Plain stream.
                Load(stream, FormatType.Txt);
            }
        }
        /// <summary>
        /// Disposes the instance.
        /// </summary>
        public void Dispose()
        {
            this.ContentChanged -= RTE_ContentChanged;
            ClearValue(HtmlTextProperty);
            ClearValue(RtfTextProperty);
            ClearValue(PlainTextProperty);
            base.Dispose();
        }
        #endregion
    }
}
